@@ -14,7 +14,7 @@ from .Base import _Base, _check_for_error
 from .Project import _Project
 from .Datasource import _Datasource
 
-RETRY_AFTER = 30 # seconds
+RETRY_AFTER = 5 # seconds
 
 
 class DynamoFL(_Base):
@@ -38,13 +38,14 @@ class DynamoFL(_Base):
         self.connect_to_ws()
 
     def _on_open(self, ws):
+        print('>>> Connection to DynamoFL established.')
         self.ws.send('{ "action": "auth", "token": "' + self.token + '" }')
 
     def _on_message(self, ws, res):
         j = json.loads(res)
         if j['event'] == 'client-info':
             self.instance_id = j['data']['id']
-            self.initiate_project_participants(should_fetch_bridges=True, should_spawn_threads=True)
+            self.initiate_project_participants(should_fetch_bridges=True, should_spawn_train=True)
 
         if j['event'] == 'new-project':
             project_key = j['data']['projectKey']
@@ -121,18 +122,18 @@ class DynamoFL(_Base):
                 print('Federation error occured:\n  ' + j['data']['errorMessage'])
 
     def connect_to_ws(self):
-        t = threading.Thread(target=self.ws.run_forever)
+        t = threading.Thread(target=self.ws.run_forever, kwargs={'reconnect': RETRY_AFTER})
         t.daemon = False
         t.start()
 
     def _on_close(self, ws, close_status_code, close_msg):
         print('Connection closed')
         print(close_msg)
-        # print(f'Trying to reestablish connection...')
 
     def _on_error(self, ws, error):
         print('Connection error:')
         print(error)
+        print(f'Will attempt to reconnect every {RETRY_AFTER} seconds...')
 
     def _get_last_fed_model_round(self, current_round, is_complete):
         if is_complete:
@@ -206,7 +207,7 @@ class DynamoFL(_Base):
             print('Done.')
             print()
 
-    def initiate_project_participants(self, should_fetch_bridges=False, should_spawn_threads=False, ds=None):
+    def initiate_project_participants(self, should_fetch_bridges=False, should_spawn_train=False, ds=None):
 
         if should_fetch_bridges:
             if ds:
@@ -224,7 +225,7 @@ class DynamoFL(_Base):
                         'trainer_key': i['trainerKey'],
                         'hyper_param_values': i['hyperParamValues']
                     })
-        if should_spawn_threads:
+        if should_spawn_train:
             for p in self.project_participants:
                 project_key = p['project_key']
                 datasource_key = p['datasource_key']
@@ -235,8 +236,13 @@ class DynamoFL(_Base):
                 So project_participants = [item1, item2]
                 We only want create a thread for items that we haven't done so already
                 '''
+                # if threads_key in self.dfl.on_round_threads:
+                #     continue
+                if ds and ds != datasource_key:
+                    continue
+
                 info = self._make_request('GET', f'/projects/{project_key}')
-                self.train_and_test_callback(datasource_key, info)    
+                self.train_and_test_callback(datasource_key, info)
                 
 
     # creates a new datasource in the api
